package com.qiyuesuo.sdk.api;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import com.qiyuesuo.sdk.exception.BaseSdkException;
import com.qiyuesuo.sdk.sign.SignInfo;
import com.qiyuesuo.sdk.sign.Stamper;
import com.qiyuesuo.sdk.signer.Company;
import com.qiyuesuo.sdk.signer.Person;
import com.qiyuesuo.sdk.verify.PdfVerifyResult;

/**
 * 本地签接口
 */

public interface LocalSignService {

	/**
	 * 平台签署本地合同(无签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf) throws BaseSdkException;
	/**
	 * 平台签署本地合同(有签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @param sealId 印章Id
	 * @param stamper 签署位置
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf,Long sealId,Stamper stamper) throws BaseSdkException;

	/**
	 * 个人用户签署本地合同(无签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @param person 用户信息
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf, Person person) throws BaseSdkException;
	
	/**
	 * 个人用户签署本地合同(有签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @param person 用户信息
	 * @param sealData 印章图片的base64编码
	 * @param stamper 签署位置
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf, Person person,String sealData,Stamper stamper) throws BaseSdkException;

	/**
	 * 公司用户签署本地合同(无签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @param company 公司信息
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf, Company company) throws BaseSdkException;
	
	/**
	 * 公司用户签署本地合同(有签名外观)
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @param company 公司信息
	 * @param sealData 印章图片的base64编码
	 * @param stamper 签署位置
	 * @param acrossPagePos 骑缝章位置
	 * @throws BaseSdkException SDK异常
	 */
	void sign(InputStream input, OutputStream outputPdf, Company company,String sealData,Stamper stamper) throws BaseSdkException;

	/**
	 * 批量签署接口(不支持自定义外观)
	 * @param input  输入pdf文件流
	 * @param outputPdf 输出pdf文件流
	 * @param infos 签署信息
	 */
	void batchSign(InputStream input, OutputStream outputPdf,List<SignInfo> infos) throws BaseSdkException;
	
	/**
	 * 完成合同签署，并对合同进行封存
	 * 
	 * @param input 输入PDF文件流
	 * @param outputPdf 输出PDF文件流
	 * @throws BaseSdkException SDK调用异常
	 */
	void complete(InputStream input, OutputStream outputPdf) throws BaseSdkException;

	/**
	 * 校验合同的有效性
	 * 
	 * @param input 输入合同文件流
	 * @return 
	 * @throws BaseSdkException SDK异常
	 */
	PdfVerifyResult verify(InputStream input) throws BaseSdkException;
}
