package com.qiyuesuo.sdk.verify;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.imageio.ImageIO;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import com.qiyuesuo.pdf.text.Rectangle;
import com.qiyuesuo.pdf.text.pdf.AcroFields;
import com.qiyuesuo.pdf.text.pdf.AcroFields.FieldPosition;
import com.qiyuesuo.pdf.text.pdf.AcroFields.Item;
import com.qiyuesuo.pdf.text.pdf.PRStream;
import com.qiyuesuo.pdf.text.pdf.PdfDictionary;
import com.qiyuesuo.pdf.text.pdf.PdfIndirectReference;
import com.qiyuesuo.pdf.text.pdf.PdfName;
import com.qiyuesuo.pdf.text.pdf.PdfReader;
import com.qiyuesuo.pdf.text.pdf.parser.PdfImageObject;
import com.qiyuesuo.pdf.text.pdf.security.CertificateInfo;
import com.qiyuesuo.pdf.text.pdf.security.CertificateInfo.X500Name;
import com.qiyuesuo.pdf.text.pdf.security.PdfPKCS7;
import com.qiyuesuo.sdk.utils.Base64Utils;
import com.qiyuesuo.sdk.utils.StringUtils;

public class PdfVerifyUtils {
	public static final BouncyCastleProvider PROVIDER = new BouncyCastleProvider();
//	private static final String QIYUESUO = "契约锁电子合同云平台";
//	private static final String FADADA = "法大大网络科技有限公司";

	
	/**
	 * 验签方法 如果签名中不包含覆盖全文的签名 那么认为签名全部被篡改
	 * @param pdfByte
	 * @return
	 * @throws Exception
	 */
	public static List<SignatureInfo> getSignatureInfos(byte[] pdfByte) throws Exception {
		ByteArrayInputStream input = new ByteArrayInputStream(pdfByte);
		return getSignatureInfos(input);
	}

	/**
	 *  验签方法 如果签名中不包含覆盖全文的签名 那么认为签名全部被篡改
	 * @param input
	 * @return
	 * @throws Exception
	 */
	public static List<SignatureInfo> getSignatureInfos(InputStream input) throws Exception {
		PdfReader reader = null;
		List<SignatureInfo> signatureInfoList = new ArrayList<>();

		// Provider provider = new BouncyCastleProvider();
		try {
			java.security.Security.addProvider(PROVIDER);
		} catch (Exception e) {
			throw e;
		}
		try {
			reader = new PdfReader(input);
			AcroFields fields = reader.getAcroFields();
			ArrayList<String> signatureNames = fields.getSignatureNames();
			if (signatureNames == null || signatureNames.isEmpty()) {
				return signatureInfoList;
			}
			// 循环验证每个签章
			for (String fieldName : signatureNames) {
				SignatureInfo signatureInfo = new SignatureInfo();

				// 签名是否覆盖全文
				boolean signatureCoversWholeDocument = fields.signatureCoversWholeDocument(fieldName);
				signatureInfo.setSignatureCoversWholeDocument(signatureCoversWholeDocument);

				// Item fieldItem = fields.getFieldItem(fieldName);
				// String[] appearanceStates = fields.getAppearanceStates(fieldName);
				// PdfIndirectReference normalAppearance = fields.getNormalAppearance(fieldName);
				// byte[] bytes = normalAppearance.getBytes();

				PdfPKCS7 pk = fields.verifySignature(fieldName);

				Field rsaDataField = PdfPKCS7.class.getDeclaredField("RSAdata");
				rsaDataField.setAccessible(true);
				Object rsaDataFieldContent = rsaDataField.get(pk);
				if (rsaDataFieldContent != null && ((byte[]) rsaDataFieldContent).length == 0) {
					rsaDataField.set(pk, null);
				}

				X509Certificate cert = pk.getSigningCertificate();

				// 证书有效期开始
				Date notBefore = cert.getNotBefore();
				signatureInfo.setCertDateFrom(notBefore);

				// 证书有效期结束
				Date notAfter = cert.getNotAfter();
				signatureInfo.setCertDateTo(notAfter);

				// 颁发机构
				X500Name issuerFields = CertificateInfo.getIssuerFields(cert);
				String organization = issuerFields.getField("O");
				String fieldCn = issuerFields.getField("CN");
				if (StringUtils.isNotBlank(fieldCn) && !fieldCn.equalsIgnoreCase(organization)) {
					organization = fieldCn + " (" + organization + ")";
				}
				signatureInfo.setOrganization(organization);

				// 签约人
				String signatory = CertificateInfo.getSubjectFields(cert).getField("CN");
				signatureInfo.setSignatory(signatory);

				/*// 设置签署平台 还未统一结构 暂时只有契约锁和法大大两个平台
				List<String> fieldArray = CertificateInfo.getSubjectFields(cert).getFieldArray("OU");
				if (fieldArray != null) {
					for (String field : fieldArray) {
						if (field.contains(FADADA)) {
							signatureInfo.setSignPlatform(FADADA);
							break;
						}
						if (field.contains(QIYUESUO)) {
							signatureInfo.setSignPlatform(QIYUESUO);
							break;
						}
					}
				}

				// 兼容契约锁平台信息
				String org = CertificateInfo.getSubjectFields(cert).getField("O");
				if (QIYUESUO.equals(org) || "QIYUESUO".equals(org)) {
					signatureInfo.setSignPlatform(QIYUESUO);
				}*/

				// 签约原因
				String reason = pk.getReason();
				signatureInfo.setSignReason(reason);

				// 签约时间
				Date signDate = pk.getSignDate() == null ? null : pk.getSignDate().getTime();
				signatureInfo.setSignTime(signDate);

				// 签名算法
				signatureInfo.setStrAlgName(pk.getHashAlgorithm());

				// 签名校验结果
				if (pk.verify()) {
					signatureInfo.setCode(0);
					signatureInfo.setMsg(SignatureInfo.SUCCESS_MSG);
				} else {
					signatureInfo.setCode(1);
					signatureInfo.setMsg(SignatureInfo.ERROR_MSG);
				}

				// 时间戳设置
				if (pk.getTimeStampDate() != null) {
					signatureInfo.setHasTimeStamp(true);
					signatureInfo.setTimeStamp(pk.getTimeStampDate().getTime());
					signatureInfo.setVerifyTimestamp(pk.verifyTimestampImprint());
				} else {
					signatureInfo.setHasTimeStamp(false);
				}

				// 获取签名外观图片
				List<FieldPosition> fps = fields.getFieldPositions(fieldName);
				if (fps != null && fps.size() > 0) {
					FieldPosition fp = fps.get(0);
					Rectangle pos = fp.position;
					if (pos.getWidth() == 0 || pos.getHeight() == 0) {
						signatureInfo.setVisibleSignature(false);
					} else {
						try {
							signatureInfo.setVisibleSignature(true);
							String sealImageString = getSealImageString(fields, fieldName);
							signatureInfo.setImgString(sealImageString);
						} catch (Exception e) {
							signatureInfo.setVisibleSignature(false);
							e.printStackTrace();
						}
					}
				}
				signatureInfoList.add(signatureInfo);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Exception e) {
				}
			}
		}

		boolean signatureCoversWholeDocument = false;
		for (SignatureInfo signatureInfo : signatureInfoList) {
			if (signatureInfo.getSignatureCoversWholeDocument()) {
				signatureCoversWholeDocument = true;
				break;
			}
		}

		if (!signatureCoversWholeDocument) {
			for (SignatureInfo signatureInfo : signatureInfoList) {
				signatureInfo.setCode(1);
				signatureInfo.setMsg(SignatureInfo.ERROR_MSG);
			}
		}
		return signatureInfoList;
	}

	/**
	 * 获取签名外观图片 返回图片BASE64字符串
	 * @param fields
	 * @param fieldName
	 * @return
	 * @throws Exception
	 */
	private static String getSealImageString(AcroFields fields, String fieldName) throws Exception {
		Item fieldItem = fields.getFieldItem(fieldName);
		PdfDictionary pdfDictionary = fieldItem.getValue(0);

		PdfDictionary signatureDR = pdfDictionary.getAsDict(PdfName.DR);
		PdfDictionary drXobject = signatureDR.getAsDict(PdfName.XOBJECT);
		PdfIndirectReference frm = drXobject.getAsIndirectObject(PdfName.FRM);
		if (frm == null) {
			throw new Exception("未找到印章图像索引");
		}

		PdfDictionary tempImageDic1 = (PdfDictionary) PdfReader.getPdfObject(frm);
		PdfDictionary pdfTempRes1 = tempImageDic1.getAsDict(PdfName.RESOURCES);
		PdfDictionary pdfTempXobject1 = pdfTempRes1.getAsDict(PdfName.XOBJECT);
		PdfIndirectReference tmpN2 = pdfTempXobject1.getAsIndirectObject(PdfName.N2);
		if (tmpN2 == null) {
			throw new Exception("未找到印章图像索引");
		}

		PdfDictionary tempImageDic2 = (PdfDictionary) PdfReader.getPdfObject(tmpN2);
		PdfDictionary pdfTempRes2 = tempImageDic2.getAsDict(PdfName.RESOURCES);
		PdfDictionary pdfTempXobject = pdfTempRes2.getAsDict(PdfName.XOBJECT);

		Set<PdfName> setPdfNames = pdfTempXobject.getKeys();
		PdfName[] pdfNames = new PdfName[setPdfNames.size()];
		PdfName[] names = setPdfNames.toArray(pdfNames);
		if (names.length == 0) {
			throw new Exception("未发现签章外观图片数据");
		}

		BufferedImage bufferedImage0 = null;
		BufferedImage bufferedImage1 = null;
		for (PdfName name : names) {
			PdfIndirectReference img = pdfTempXobject.getAsIndirectObject(name);
			PRStream stream = (PRStream) PdfReader.getPdfObject(img);
			Set<PdfName> keys = stream.getKeys();
			PdfImageObject image = new PdfImageObject(stream);
			if (!keys.contains(PdfName.SMASK)) {
				bufferedImage0 = image.getBufferedImage();
			}
			if (keys.contains(PdfName.SMASK)) {
				bufferedImage1 = image.getBufferedImage();
			}
		}

		BufferedImage imageBuffer = null;
		if (bufferedImage0 == null && bufferedImage1 == null) {
			throw new Exception("未发现签章外观图片数据");
		} else if (bufferedImage0 != null && bufferedImage1 == null) {
			imageBuffer = bufferedImage0;
		} else if (bufferedImage0 == null && bufferedImage1 != null) {
			imageBuffer = bufferedImage1;
		} else {
			imageBuffer = transferAlpha(bufferedImage0, bufferedImage1);
		}

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ImageIO.write(imageBuffer, "png", out);
		byte[] byteArray = out.toByteArray();
		String string = Base64Utils.encode(byteArray);
		return "data:image/png;base64," + string;
	}

	private static BufferedImage transferAlpha(BufferedImage image0, BufferedImage image1) {
		try {
			// int hashCode = image0.hashCode();
			// ImageIO.write(image0, "png", new File("D:/test1/"+hashCode+"_image0.png"));
			// ImageIO.write(image1, "png", new File("D:/test1/"+hashCode+"_image1.png"));
			// 将非黑色内容透明
			BufferedImage bufferedImage = new BufferedImage(image0.getWidth(), image0.getHeight(), BufferedImage.TYPE_4BYTE_ABGR);
			for (int j1 = bufferedImage.getMinY(); j1 < bufferedImage.getHeight(); j1++) {
				for (int j2 = bufferedImage.getMinX(); j2 < bufferedImage.getWidth(); j2++) {
					int rgb = image0.getRGB(j2, j1);
					int R = (rgb & 0xff0000) >> 16;
					int G = (rgb & 0xff00) >> 8;
					int B = (rgb & 0xff);

					if (!(((255 - R) > 200) && ((255 - G) > 200) && ((255 - B) > 200))) {
						bufferedImage.setRGB(j2, j1, image1.getRGB(j2, j1));
					}
				}
			}
			// ImageIO.write(bufferedImage, "png", new File("D:/test1/"+hashCode+"_image3.png"));
			return bufferedImage;
		} catch (Exception e) {
			return null;
		}
	}

}
